@@ -100,7 +100,7 @@ waitpid(
 	} else {
 	    prevPtr->nextPtr = waitPtr->nextPtr;
 	}
-	ckfree((char *) waitPtr);
+	Tcl_Free(waitPtr);
 	return result;
     }
 
@@ -156,7 +156,11 @@ waitpid(
 		goto waitAgain;
 	    }
 	}
-	waitPtr = (WaitInfo *) ckalloc(sizeof(WaitInfo));
+	waitPtr = (WaitInfo *) Tcl_AttemptAlloc(sizeof(WaitInfo));
+	if (!waitPtr) {
+	    errno = ENOMEM;
+	    return -1;
+	}
 	waitPtr->pid = result;
 	waitPtr->status = status;
 	waitPtr->nextPtr = deadList;
