@@ -6,9 +6,9 @@
  *	that don't exactly fit are passed up to the next larger size. Blocks
  *	over a certain size are directly allocated from the system.
  *
- * Copyright (c) 1983 Regents of the University of California.
- * Copyright (c) 1996-1997 Sun Microsystems, Inc.
- * Copyright (c) 1998-1999 by Scriptics Corporation.
+ * Copyright © 1983 Regents of the University of California.
+ * Copyright © 1996-1997 Sun Microsystems, Inc.
+ * Copyright © 1998-1999 Scriptics Corporation.
  *
  * Portions contributed by Chris Kingsley, Jack Jansen and Ray Johnson.
  *
@@ -22,7 +22,7 @@
  */
 
 #include "tclInt.h"
-#if !defined(TCL_THREADS) || !defined(USE_THREAD_ALLOC)
+#if !TCL_THREADS || !defined(USE_THREAD_ALLOC)
 
 #if defined(USE_TCLALLOC) && USE_TCLALLOC
 
@@ -31,8 +31,8 @@
  * until Tcl uses config.h properly.
  */
 
-#if defined(_MSC_VER) || defined(__MSVCRT__) || defined(__BORLANDC__)
-typedef unsigned long caddr_t;
+#if defined(_MSC_VER) || defined(__MSVCRT__)
+typedef size_t caddr_t;
 #endif
 
 /*
@@ -47,17 +47,18 @@ typedef unsigned long caddr_t;
  */
 
 union overhead {
-    union overhead *next;		/* when free */
-    unsigned char padding[TCL_ALLOCALIGN];	/* align struct to TCL_ALLOCALIGN bytes */
+    union overhead *next;	/* when free */
+    unsigned char padding[TCL_ALLOCALIGN];
+				/* align struct to TCL_ALLOCALIGN bytes */
     struct {
-	unsigned char magic0;		/* magic number */
-	unsigned char index;		/* bucket # */
-	unsigned char unused;		/* unused */
-	unsigned char magic1;		/* other magic number */
+	unsigned char magic0;	/* magic number */
+	unsigned char index;	/* bucket # */
+	unsigned char unused;	/* unused */
+	unsigned char magic1;	/* other magic number */
 #ifndef NDEBUG
-	unsigned short rmagic;		/* range magic number */
-	unsigned long size;		/* actual block size */
-	unsigned short unused2;		/* padding to 8-byte align */
+	unsigned short rmagic;	/* range magic number */
+	size_t size;		/* actual block size */
+	unsigned short unused2;	/* padding to 8-byte align */
 #endif
     } ovu;
 #define overMagic0	ovu.magic0
@@ -67,7 +68,6 @@ union overhead {
 #define realBlockSize	ovu.size
 };
 
-
 #define MAGIC		0xEF	/* magic # on accounting info */
 #define RMAGIC		0x5555	/* magic # on range info */
 
@@ -92,9 +92,10 @@ union overhead {
  * precedes the data area returned to the user.
  */
 
-#define MINBLOCK	((sizeof(union overhead) + (TCL_ALLOCALIGN-1)) & ~(TCL_ALLOCALIGN-1))
+#define MINBLOCK \
+    ((sizeof(union overhead) + (TCL_ALLOCALIGN-1)) & ~(TCL_ALLOCALIGN-1))
 #define NBUCKETS	(13 - (MINBLOCK >> 4))
-#define MAXMALLOC	(1<<(NBUCKETS+2))
+#define MAXMALLOC	((size_t)1 << (NBUCKETS+2))
 static union overhead *nextf[NBUCKETS];
 
 /*
@@ -116,12 +117,12 @@ static struct block bigBlocks={	/* Big blocks aren't suballocated. */
 
 /*
  * The allocator is protected by a special mutex that must be explicitly
- * initialized. Futhermore, because Tcl_Alloc may be used before anything else
+ * initialized. Furthermore, because Tcl_Alloc may be used before anything else
  * in Tcl, we make this module self-initializing after all with the allocInit
  * variable.
  */
 
-#ifdef TCL_THREADS
+#if TCL_THREADS
 static Tcl_Mutex *allocMutexPtr;
 #endif
 static int allocInit = 0;
@@ -133,7 +134,7 @@ static int allocInit = 0;
  * a given block size.
  */
 
-static	unsigned int numMallocs[NBUCKETS+1];
+static	size_t numMallocs[NBUCKETS+1];
 #endif
 
 #if !defined(NDEBUG)
@@ -148,7 +149,7 @@ static	unsigned int numMallocs[NBUCKETS+1];
  * Prototypes for functions used only in this file.
  */
 
-static void		MoreCore(int bucket);
+static void		MoreCore(size_t bucket);
 
 /*
  *-------------------------------------------------------------------------
@@ -171,7 +172,7 @@ TclInitAlloc(void)
 {
     if (!allocInit) {
 	allocInit = 1;
-#ifdef TCL_THREADS
+#if TCL_THREADS
 	allocMutexPtr = Tcl_GetAllocMutex();
 #endif
     }
@@ -249,13 +250,13 @@ TclFinalizeAllocSubsystem(void)
  *----------------------------------------------------------------------
  */
 
-char *
+void *
 TclpAlloc(
-    unsigned int numBytes)	/* Number of bytes to allocate. */
+    size_t numBytes)		/* Number of bytes to allocate. */
 {
     union overhead *overPtr;
-    long bucket;
-    unsigned amount;
+    size_t bucket;
+    size_t amount;
     struct block *bigBlockPtr = NULL;
 
     if (!allocInit) {
@@ -274,8 +275,8 @@ TclpAlloc(
 
     if (numBytes >= MAXMALLOC - OVERHEAD) {
 	if (numBytes <= UINT_MAX - OVERHEAD -sizeof(struct block)) {
-	    bigBlockPtr = (struct block *) TclpSysAlloc(
-		    (sizeof(struct block) + OVERHEAD + numBytes), 0);
+	    bigBlockPtr = TclpSysAlloc(
+		    sizeof(struct block) + OVERHEAD + numBytes);
 	}
 	if (bigBlockPtr == NULL) {
 	    Tcl_MutexUnlock(allocMutexPtr);
@@ -385,12 +386,12 @@ TclpAlloc(
 
 static void
 MoreCore(
-    int bucket)			/* What bucket to allocat to. */
+    size_t bucket)		/* What bucket to allocate to. */
 {
     union overhead *overPtr;
-    long size;		/* size of desired block */
-    long amount;		/* amount to allocate */
-    int numBlocks;		/* how many blocks we get */
+    size_t size;		/* size of desired block */
+    size_t amount;		/* amount to allocate */
+    size_t numBlocks;		/* how many blocks we get */
     struct block *blockPtr;
 
     /*
@@ -398,15 +399,14 @@ MoreCore(
      * VAX, I think) or for a negative arg.
      */
 
-    size = 1 << (bucket + 3);
+    size = ((size_t)1) << (bucket + 3);
     ASSERT(size > 0);
 
     amount = MAXMALLOC;
     numBlocks = amount / size;
     ASSERT(numBlocks*size == amount);
 
-    blockPtr = (struct block *) TclpSysAlloc(
-	    (sizeof(struct block) + amount), 1);
+    blockPtr = TclpSysAlloc(sizeof(struct block) + amount);
     /* no more room! */
     if (blockPtr == NULL) {
 	return;
@@ -446,9 +446,9 @@ MoreCore(
 
 void
 TclpFree(
-    char *oldPtr)		/* Pointer to memory to free. */
+    void *oldPtr)		/* Pointer to memory to free. */
 {
-    long size;
+    size_t size;
     union overhead *overPtr;
     struct block *bigBlockPtr;
 
@@ -509,16 +509,16 @@ TclpFree(
  *----------------------------------------------------------------------
  */
 
-char *
+void *
 TclpRealloc(
-    char *oldPtr,		/* Pointer to alloced block. */
-    unsigned int numBytes)	/* New size of memory. */
+    void *oldPtr,		/* Pointer to alloc'ed block. */
+    size_t numBytes)		/* New size of memory. */
 {
     int i;
     union overhead *overPtr;
     struct block *bigBlockPtr;
     int expensive;
-    unsigned long maxSize;
+    size_t maxSize;
 
     if (oldPtr == NULL) {
 	return TclpAlloc(numBytes);
@@ -581,9 +581,9 @@ TclpRealloc(
 #endif
 
 	Tcl_MutexUnlock(allocMutexPtr);
-	return (char *)(overPtr+1);
+	return (void *)(overPtr+1);
     }
-    maxSize = 1 << (i+3);
+    maxSize = (size_t)1 << (i+3);
     expensive = 0;
     if (numBytes+OVERHEAD > maxSize) {
 	expensive = 1;
@@ -610,7 +610,7 @@ TclpRealloc(
     }
 
     /*
-     * Ok, we don't have to copy, it fits as-is
+     * No need to copy. It fits as-is.
      */
 
 #ifndef NDEBUG
@@ -645,29 +645,29 @@ void
 mstats(
     char *s)			/* Where to write info. */
 {
-    int i, j;
+    unsigned int i, j;
     union overhead *overPtr;
-    int totalFree = 0, totalUsed = 0;
+    size_t totalFree = 0, totalUsed = 0;
 
     Tcl_MutexLock(allocMutexPtr);
 
     fprintf(stderr, "Memory allocation statistics %s\nTclpFree:\t", s);
     for (i = 0; i < NBUCKETS; i++) {
 	for (j=0, overPtr=nextf[i]; overPtr; overPtr=overPtr->next, j++) {
-	    fprintf(stderr, " %d", j);
+	    fprintf(stderr, " %u", j);
 	}
-	totalFree += j * (1 << (i + 3));
+	totalFree += ((size_t)j) * ((size_t)1 << (i + 3));
     }
 
     fprintf(stderr, "\nused:\t");
     for (i = 0; i < NBUCKETS; i++) {
-	fprintf(stderr, " %d", numMallocs[i]);
-	totalUsed += numMallocs[i] * (1 << (i + 3));
+	fprintf(stderr, " %" TCL_Z_MODIFIER "u", numMallocs[i]);
+	totalUsed += numMallocs[i] * ((size_t)1 << (i + 3));
     }
 
-    fprintf(stderr, "\n\tTotal small in use: %d, total free: %d\n",
-	    totalUsed, totalFree);
-    fprintf(stderr, "\n\tNumber of big (>%d) blocks in use: %d\n",
+    fprintf(stderr, "\n\tTotal small in use: %" TCL_Z_MODIFIER "u, total free: %" TCL_Z_MODIFIER "u\n",
+	totalUsed, totalFree);
+    fprintf(stderr, "\n\tNumber of big (>%" TCL_Z_MODIFIER "u) blocks in use: %" TCL_Z_MODIFIER "u\n",
 	    MAXMALLOC, numMallocs[NBUCKETS]);
 
     Tcl_MutexUnlock(allocMutexPtr);
@@ -692,11 +692,12 @@ mstats(
  *----------------------------------------------------------------------
  */
 
-char *
+#undef TclpAlloc
+void *
 TclpAlloc(
-    unsigned int numBytes)	/* Number of bytes to allocate. */
+    size_t numBytes)	/* Number of bytes to allocate. */
 {
-    return (char *) malloc(numBytes);
+    return malloc(numBytes);
 }
 
 /*
@@ -715,9 +716,10 @@ TclpAlloc(
  *----------------------------------------------------------------------
  */
 
+#undef TclpFree
 void
 TclpFree(
-    char *oldPtr)		/* Pointer to memory to free. */
+    void *oldPtr)		/* Pointer to memory to free. */
 {
     free(oldPtr);
     return;
@@ -739,15 +741,17 @@ TclpFree(
  *----------------------------------------------------------------------
  */
 
-char *
+void *
 TclpRealloc(
-    char *oldPtr,		/* Pointer to alloced block. */
-    unsigned int numBytes)	/* New size of memory. */
+    void *oldPtr,		/* Pointer to alloced block. */
+    size_t numBytes)		/* New size of memory. */
 {
-    return (char *) realloc(oldPtr, numBytes);
+    return realloc(oldPtr, numBytes);
 }
 
 #endif /* !USE_TCLALLOC */
+#else
+TCL_MAC_EMPTY_FILE(generic_tclAlloc_c)
 #endif /* !TCL_THREADS */
 
 /*
