@@ -25,6 +25,8 @@ static void		ClockFmtObj_UpdateString(Tcl_Obj *objPtr);
 static Tcl_HashEntry *	ClockFmtScnStorageAllocProc(Tcl_HashTable *, void *keyPtr);
 static void		ClockFmtScnStorageFreeProc(Tcl_HashEntry *hPtr);
 static void		ClockFmtScnStorageDelete(ClockFmtScnStorage *fss);
+static Tcl_Obj *	ClockFrmObjGetLocFmtKey(Tcl_Interp *interp, Tcl_Obj *objPtr);
+static Tcl_Obj *	ClockLocalizeFormat(ClockFmtScnCmdArgs *opts);
 
 TCL_DECLARE_MUTEX(ClockFmtMutex);	/* Serializes access to common format list. */
 
@@ -951,7 +953,7 @@ ClockLocalizeFormat(
     Tcl_IncrRefCount(keyObj);
 
     if (opts->mcDictObj == NULL) {
-	ClockMCDict(opts);
+	TclClockMCDict(opts);
 	if (opts->mcDictObj == NULL) {
 	    goto done;
 	}
@@ -1270,7 +1272,7 @@ LocaleListSearch(
     Tcl_Obj *valObj;
 
     /* get msgcat value */
-    valObj = ClockMCGet(opts, mcKey);
+    valObj = TclClockMCGet(opts, mcKey);
     if (valObj == NULL) {
 	return TCL_ERROR;
     }
@@ -1312,7 +1314,7 @@ ClockMCGetListIdxTree(
     int mcKey)
 {
     TclStrIdxTree *idxTree;
-    Tcl_Obj *objPtr = ClockMCGetIdx(opts, mcKey);
+    Tcl_Obj *objPtr = TclClockMCGetIdx(opts, mcKey);
 
     if (objPtr != NULL
 	    && (idxTree = TclStrIdxTreeGetFromObj(objPtr)) != NULL) {
@@ -1329,7 +1331,7 @@ ClockMCGetListIdxTree(
 	    goto done;	/* unexpected, but ...*/
 	}
 
-	valObj = ClockMCGet(opts, mcKey);
+	valObj = TclClockMCGet(opts, mcKey);
 	if (valObj == NULL) {
 	    goto done;
 	}
@@ -1340,7 +1342,7 @@ ClockMCGetListIdxTree(
 	    goto done;
 	}
 
-	ClockMCSetIdx(opts, mcKey, objPtr);
+	TclClockMCSetIdx(opts, mcKey, objPtr);
 	objPtr = NULL;
     }
 
@@ -1381,7 +1383,7 @@ ClockMCGetMultiListIdxTree(
     int *mcKeys)
 {
     TclStrIdxTree * idxTree;
-    Tcl_Obj *objPtr = ClockMCGetIdx(opts, mcKey);
+    Tcl_Obj *objPtr = TclClockMCGetIdx(opts, mcKey);
 
     if (objPtr != NULL
 	    && (idxTree = TclStrIdxTreeGetFromObj(objPtr)) != NULL) {
@@ -1399,7 +1401,7 @@ ClockMCGetMultiListIdxTree(
 	}
 
 	while (*mcKeys) {
-	    valObj = ClockMCGet(opts, *mcKeys);
+	    valObj = TclClockMCGet(opts, *mcKeys);
 	    if (valObj == NULL) {
 		goto done;
 	    }
@@ -1412,7 +1414,7 @@ ClockMCGetMultiListIdxTree(
 	    mcKeys++;
 	}
 
-	ClockMCSetIdx(opts, mcKey, objPtr);
+	TclClockMCSetIdx(opts, mcKey, objPtr);
 	objPtr = NULL;
     }
 
@@ -1661,8 +1663,8 @@ ClockScnToken_amPmInd_Proc(
 
     DetermineGreedySearchLen(opts, info, tok, &minLen, &maxLen);
 
-    amPmObj[0] = ClockMCGet(opts, MCLIT_AM);
-    amPmObj[1] = ClockMCGet(opts, MCLIT_PM);
+    amPmObj[0] = TclClockMCGet(opts, MCLIT_AM);
+    amPmObj[1] = TclClockMCGet(opts, MCLIT_PM);
 
     if (amPmObj[0] == NULL || amPmObj[1] == NULL) {
 	return TCL_ERROR;
@@ -1696,8 +1698,8 @@ ClockScnToken_LocaleERA_Proc(
 
     DetermineGreedySearchLen(opts, info, tok, &minLen, &maxLen);
 
-    eraObj[0] = ClockMCGet(opts, MCLIT_BCE);
-    eraObj[1] = ClockMCGet(opts, MCLIT_CE);
+    eraObj[0] = TclClockMCGet(opts, MCLIT_BCE);
+    eraObj[1] = TclClockMCGet(opts, MCLIT_CE);
     eraObj[2] = dataPtr->mcLiterals[MCLIT_BCE2];
     eraObj[3] = dataPtr->mcLiterals[MCLIT_CE2];
     eraObj[4] = dataPtr->mcLiterals[MCLIT_BCE3];
@@ -1903,7 +1905,7 @@ ClockScnToken_TimeZone_Proc(
     /* try to apply new time zone */
     Tcl_IncrRefCount(tzObjStor);
 
-    opts->timezoneObj = ClockSetupTimeZone(opts->dataPtr, opts->interp,
+    opts->timezoneObj = TclClockSetupTimeZone(opts->dataPtr, opts->interp,
 	    tzObjStor);
 
     Tcl_DecrRefCount(tzObjStor);
@@ -1977,7 +1979,7 @@ ClockScnToken_StarDate_Proc(
     yydate.isBce = 0;
     yydate.gregorian = 1;
 
-    if (IsGregorianLeapYear(&yydate)) {
+    if (TclIsGregorianLeapYear(&yydate)) {
 	fractYear *= 366;
     } else {
 	fractYear *= 365;
@@ -1987,7 +1989,7 @@ ClockScnToken_StarDate_Proc(
 	yydate.dayOfYear++;
     }
 
-    GetJulianDayFromEraYearDay(&yydate, GREGORIAN_CHANGE_DATE);
+    TclGetJulianDayFromEraYearDay(&yydate, GREGORIAN_CHANGE_DATE);
 
     yydate.localSeconds =
 	    -210866803200LL
@@ -2172,7 +2174,7 @@ EstimateTokenCount(
 /*
  *----------------------------------------------------------------------
  */
-ClockFmtScnStorage *
+static ClockFmtScnStorage *
 ClockGetOrParseScanFormat(
     Tcl_Interp *interp,		/* Tcl interpreter */
     Tcl_Obj *formatObj)		/* Format container */
@@ -2379,7 +2381,7 @@ ClockGetOrParseScanFormat(
  *----------------------------------------------------------------------
  */
 int
-ClockScan(
+TclClockScan(
     DateInfo *info,		/* Date fields used for parsing & converting */
     Tcl_Obj *strObj,		/* String containing the time to scan */
     ClockFmtScnCmdArgs *opts)	/* Command options */
@@ -2698,7 +2700,7 @@ ClockScan(
 
 	if (flags & CLF_TIME) {
 	    info->flags |= CLF_ASSEMBLE_SECONDS;
-	    yySecondOfDay = ToSeconds(yyHour, yyMinutes,
+	    yySecondOfDay = TclToSeconds(yyHour, yyMinutes,
 		    yySeconds, yyMeridian);
 	} else if (!(flags & (CLF_LOCALSEC | CLF_POSIXSEC))) {
 	    info->flags |= CLF_ASSEMBLE_SECONDS;
@@ -2794,9 +2796,9 @@ ClockFmtToken_AMPM_Proc(
     Tcl_Size len;
 
     if (*val < (SECONDS_PER_DAY / 2)) {
-	mcObj = ClockMCGet(opts, MCLIT_AM);
+	mcObj = TclClockMCGet(opts, MCLIT_AM);
     } else {
-	mcObj = ClockMCGet(opts, MCLIT_PM);
+	mcObj = TclClockMCGet(opts, MCLIT_PM);
     }
     if (mcObj == NULL) {
 	return TCL_ERROR;
@@ -2826,7 +2828,7 @@ ClockFmtToken_StarDate_Proc(
     int v = dateFmt->date.dayOfYear - 1;
 
     /* Convert day of year to a fractional year */
-    if (IsGregorianLeapYear(&dateFmt->date)) {
+    if (TclIsGregorianLeapYear(&dateFmt->date)) {
 	fractYear = 1000 * v / 366;
     } else {
 	fractYear = 1000 * v / 365;
@@ -2973,7 +2975,7 @@ ClockFmtToken_TimeZone_Proc(
 	Tcl_Size len;
 
 	/* convert seconds to local seconds to obtain tzName object */
-	if (ConvertUTCToLocal(opts->dataPtr, opts->interp,
+	if (TclConvertUTCToLocal(opts->dataPtr, opts->interp,
 		&dateFmt->date, opts->timezoneObj,
 		GREGORIAN_CHANGE_DATE) != TCL_OK) {
 	    return TCL_ERROR;
@@ -3001,9 +3003,9 @@ ClockFmtToken_LocaleERA_Proc(
     Tcl_Size len;
 
     if (dateFmt->date.isBce) {
-	mcObj = ClockMCGet(opts, MCLIT_BCE);
+	mcObj = TclClockMCGet(opts, MCLIT_BCE);
     } else {
-	mcObj = ClockMCGet(opts, MCLIT_CE);
+	mcObj = TclClockMCGet(opts, MCLIT_CE);
     }
     if (mcObj == NULL) {
 	return TCL_ERROR;
@@ -3029,7 +3031,7 @@ ClockFmtToken_LocaleERAYear_Proc(
     Tcl_Obj **rowv;
 
     if (dateFmt->localeEra == NULL) {
-	Tcl_Obj *mcObj = ClockMCGet(opts, MCLIT_LOCALE_ERAS);
+	Tcl_Obj *mcObj = TclClockMCGet(opts, MCLIT_LOCALE_ERAS);
 	if (mcObj == NULL) {
 	    return TCL_ERROR;
 	}
@@ -3037,7 +3039,7 @@ ClockFmtToken_LocaleERAYear_Proc(
 	    return TCL_ERROR;
 	}
 	if (rowc != 0) {
-	    dateFmt->localeEra = LookupLastTransition(opts->interp,
+	    dateFmt->localeEra = TclClockLookupLastTransition(opts->interp,
 		    dateFmt->date.localSeconds, rowc, rowv, NULL);
 	}
 	if (dateFmt->localeEra == NULL) {
@@ -3079,7 +3081,7 @@ ClockFmtToken_LocaleERAYear_Proc(
 	    /* if year in locale numerals */
 	    if (*val >= 0 && *val < 100) {
 		/* year as integer */
-		Tcl_Obj * mcObj = ClockMCGet(opts, MCLIT_LOCALE_NUMERALS);
+		Tcl_Obj * mcObj = TclClockMCGet(opts, MCLIT_LOCALE_NUMERALS);
 		if (mcObj == NULL) {
 		    return TCL_ERROR;
 		}
@@ -3254,7 +3256,7 @@ static const ClockFormatTokenMap FmtWordTokenMap = {
 /*
  *----------------------------------------------------------------------
  */
-ClockFmtScnStorage *
+static ClockFmtScnStorage *
 ClockGetOrParseFmtFormat(
     Tcl_Interp *interp,		/* Tcl interpreter */
     Tcl_Obj *formatObj)		/* Format container */
@@ -3403,7 +3405,7 @@ ClockGetOrParseFmtFormat(
  *----------------------------------------------------------------------
  */
 int
-ClockFormat(
+TclClockFormat(
     DateFormat *dateFmt,	/* Date fields used for parsing & converting */
     ClockFmtScnCmdArgs *opts)	/* Command options */
 {
@@ -3476,7 +3478,7 @@ ClockFormat(
 		}
 	    } else {
 		const char *s;
-		Tcl_Obj * mcObj = ClockMCGet(opts, PTR2INT(map->data) /* mcKey */);
+		Tcl_Obj * mcObj = TclClockMCGet(opts, PTR2INT(map->data) /* mcKey */);
 
 		if (mcObj == NULL) {
 		    goto error;
@@ -3579,7 +3581,7 @@ ClockFormat(
 
 
 void
-ClockFrmScnClearCaches(void)
+TclClockFrmScnClearCaches(void)
 {
     Tcl_MutexLock(&ClockFmtMutex);
     /* clear caches ... */
@@ -3587,7 +3589,7 @@ ClockFrmScnClearCaches(void)
 }
 
 void
-ClockFrmScnFinalize(void)
+TclClockFrmScnFinalize(void)
 {
     if (!initialized) {
 	return;
