@@ -16,7 +16,6 @@
 
 #include "tclInt.h"
 #include "tclCompile.h"
-#include <assert.h>
 
 /*
  * Prototypes for procedures defined later in this file:
@@ -2008,14 +2007,11 @@ TclCompileRegexpCmd(
 	    return TCL_ERROR;
 	}
 
-	if (len == 0) {
-	    /*
-	     * The semantics of regexp are always match on re == "".
-	     */
-
-	    PushStringLiteral(envPtr, "1");
-	    return TCL_OK;
-	}
+	/*
+	 * Note: do not optimize for len == 0, as error should be generated
+	 * at runtime if operand is not a resolvable variable.
+	 * Bug https://core.tcl-lang.org/tcl/info/cb03e57a7b24d22c
+	 */
 
 	/*
 	 * Attempt to convert pattern to glob.  If successful, push the
@@ -2201,6 +2197,7 @@ TclCompileRegsubCmd(
 		 * but we definitely can't handle that at all.
 		 */
 	    }
+	    TCL_FALLTHROUGH();
 	case '\0': case '?': case '[': case '\\':
 	    goto done;
 	}
