@@ -1102,6 +1102,9 @@ static const struct {
 #ifdef STATIC_BUILD
 	    ".static"
 #endif
+#if (defined(__MSVCRT__) || defined(_UCRT)) && (!defined(__USE_MINGW_ANSI_STDIO) || __USE_MINGW_ANSI_STDIO)
+	    ".stdio-mingw"
+#endif
 #ifndef TCL_WITH_EXTERNAL_TOMMATH
 	    ".tommath-0103"
 #endif
