@@ -156,13 +156,14 @@ Tcl_LoadObjCmd(
 		&index) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	++objv; --objc;
+	++objv;
+	--objc;
 	if (LOAD_GLOBAL == index) {
 	    flags |= TCL_LOAD_GLOBAL;
 	} else if (LOAD_LAZY == index) {
 	    flags |= TCL_LOAD_LAZY;
 	} else {
-		break;
+	    break;
 	}
     }
     if ((objc < 2) || (objc > 4)) {
@@ -482,11 +483,10 @@ Tcl_LoadObjCmd(
 	Interp *iPtr = (Interp *) target;
 	if (iPtr->legacyResult && *(iPtr->legacyResult) && !iPtr->legacyFreeProc) {
 	    /*
-	     * A call to Tcl_InitStubs() determined the caller extension and
-	     * this interp are incompatible in their stubs mechanisms, and
-	     * recorded the error in the oldest legacy place we have to do so.
+	     * A call to Tcl_InitStubs() determined the caller extension
+	     * Stubs were introduced in Tcl 8.1, so there's only one possible reason.
 	     */
-	    Tcl_SetObjResult(target, Tcl_NewStringObj(iPtr->legacyResult, -1));
+	    Tcl_SetObjResult(target, Tcl_NewStringObj("this extension is compiled for Tcl 8.x", -1));
 	    iPtr->legacyResult = NULL;
 	    iPtr->legacyFreeProc = (void (*) (void))-1;
 	}
@@ -599,6 +599,8 @@ Tcl_UnloadObjCmd(
 	case UNLOAD_LAST:		/* -- */
 	    i++;
 	    goto endOfForLoop;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
   endOfForLoop:
