@@ -485,6 +485,8 @@ InfoObjectIsACmd(
 	    return TCL_ERROR;
 	}
 	break;
+    default:
+	TCL_UNREACHABLE();
     }
 
     /*
@@ -538,6 +540,8 @@ InfoObjectIsACmd(
 	    result = TclOOIsReachable(o2Ptr->classPtr, oPtr->selfCls);
 	}
 	break;
+    default:
+	TCL_UNREACHABLE();
     }
     Tcl_SetObjResult(interp, Tcl_NewBooleanObj(result));
     return TCL_OK;
@@ -628,6 +632,8 @@ InfoObjectMethodsCmd(
 		    return TCL_ERROR;
 		}
 		break;
+	    default:
+		TCL_UNREACHABLE();
 	    }
 	}
     }
@@ -888,7 +894,7 @@ InfoObjectVariablesCmd(
 	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 		    "option \"%s\" is not exactly \"-private\"",
 		    TclGetString(objv[2])));
-	    Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_ARG");
+	    OO_ERROR(interp, BAD_ARG);
 	    return TCL_ERROR;
 	}
 	isPrivate = 1;
@@ -1015,7 +1021,7 @@ InfoClassConstrCmd(
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		"definition not available for this kind of method",
 		TCL_AUTO_LENGTH));
-	Tcl_SetErrorCode(interp, "TCL", "OO", "METHOD_TYPE", (char *)NULL);
+	OO_ERROR(interp, METHOD_TYPE);
 	return TCL_ERROR;
     }
 
@@ -1145,9 +1151,9 @@ InfoClassDefnNsCmd(
 	return TCL_ERROR;
     }
 
-    if (kind) {
+    if (kind) {			// -instance
 	nsNamePtr = clsPtr->objDefinitionNs;
-    } else {
+    } else {			// -class
 	nsNamePtr = clsPtr->clsDefinitionNs;
     }
     if (nsNamePtr) {
@@ -1193,7 +1199,7 @@ InfoClassDestrCmd(
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		"definition not available for this kind of method",
 		TCL_AUTO_LENGTH));
-	Tcl_SetErrorCode(interp, "TCL", "OO", "METHOD_TYPE", (char *)NULL);
+	OO_ERROR(interp, METHOD_TYPE);
 	return TCL_ERROR;
     }
 
@@ -1412,6 +1418,8 @@ InfoClassMethodsCmd(
 		    return TCL_ERROR;
 		}
 		break;
+	    default:
+		TCL_UNREACHABLE();
 	    }
 	}
     }
@@ -1427,6 +1435,8 @@ InfoClassMethodsCmd(
 	case SCOPE_UNEXPORTED:
 	    flag = 0;
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
 
@@ -1690,7 +1700,7 @@ InfoClassVariablesCmd(
 	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 		    "option \"%s\" is not exactly \"-private\"",
 		    TclGetString(objv[2])));
-	    Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_ARG");
+	    OO_ERROR(interp, BAD_ARG);
 	    return TCL_ERROR;
 	}
 	isPrivate = 1;
@@ -1756,7 +1766,7 @@ InfoObjectCallCmd(
     if (contextPtr == NULL) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		"cannot construct any call chain", TCL_AUTO_LENGTH));
-	Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_CALL_CHAIN");
+	OO_ERROR(interp, BAD_CALL_CHAIN);
 	return TCL_ERROR;
     }
     Tcl_SetObjResult(interp,
@@ -1802,7 +1812,7 @@ InfoClassCallCmd(
     if (callPtr == NULL) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		"cannot construct any call chain", TCL_AUTO_LENGTH));
-	Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_CALL_CHAIN");
+	OO_ERROR(interp, BAD_CALL_CHAIN);
 	return TCL_ERROR;
     }
     Tcl_SetObjResult(interp, TclOORenderCallChain(interp, callPtr));
