@@ -13,7 +13,6 @@
 
 #include "tclInt.h"
 #include "tclFileSystem.h"
-#include <assert.h>
 
 /*
  * Prototypes for functions defined later in this file.
@@ -679,10 +678,7 @@ TclPathPart(
 		}
 	    }
 	    default:
-		/* We should never get here */
-		Tcl_Panic("Bad portion to TclPathPart");
-		/* For less clever compilers */
-		return NULL;
+		TCL_UNREACHABLE();
 	    }
 	} else if (fsPathPtr->cwdPtr != NULL) {
 	    /* Relative path */
