@@ -60,24 +60,49 @@ Tcl_InitStubs(
     const char *actualVersion = NULL;
     void *pkgData = NULL;
     const TclStubs *stubsPtr = iPtr->stubTable;
-    const char *tclName = (((exact&0xFF00) >= 0x900) ? "tcl" : "Tcl");
+    const char *tclName = "tcl";
 
-#undef TCL_STUB_MAGIC /* We need the TCL_STUB_MAGIC from Tcl 8.x here */
-#define TCL_STUB_MAGIC		((int) 0xFCA3BACF)
+    if ((exact&0xFF00) < 0x900) {
+	magic = (int)0xFCA3BACF; /* TCL_STUB_MAGIC from Tcl 8.x */
+	tclName = "Tcl";
+    }
     /*
      * We can't optimize this check by caching tclStubsPtr because that
      * prevents apps from being able to load/unload Tcl dynamically multiple
      * times. [Bug 615304]
      */
 
-    if (!stubsPtr || (stubsPtr->magic != (((exact&0xFF00) >= 0x900) ? magic : TCL_STUB_MAGIC))) {
-	iPtr->legacyResult = "interpreter uses an incompatible stubs mechanism";
-	iPtr->legacyFreeProc = 0; /* TCL_STATIC */
+    if (!stubsPtr || (stubsPtr->magic != magic)) {
+	exact &= 0xFFFF00; /* Filter out minor/major Tcl version */
+	if (!exact) {
+	    exact = 0x060800;
+	}
+	if (stubsPtr && (stubsPtr->magic == TCL_STUB_MAGIC)
+		&& ((exact|0x010000) == 0x070800)) {
+	    /* We are running in Tcl 9.x, but extension is compiled with 8.6 */
+	    stubsPtr->tcl_SetObjResult(interp, stubsPtr->tcl_ObjPrintf(
+		    "this extension is compiled for Tcl %d.%d",
+		    (exact & 0x0FF00)>>8, (exact & 0x0FF0000)>>16));
+	} else if (stubsPtr && (stubsPtr->magic == (int)0xFCA3BACF)
+		&& ((exact & 0x0FF00) >= 0x0900)) {
+	    /* We are running in Tcl 8.x, but extension is compiled with 9.0+ */
+	    char major[4], minor[4];
+	    snprintf(major, sizeof(major), "%d", (exact & 0xFF00)>>8);
+	    snprintf(minor, sizeof(minor), "%d", (exact & 0xFF0000)>>16);
+	    stubsPtr->tcl_AppendResult(interp,
+		    "this extension is compiled for Tcl ", major, ".", minor, (char *)NULL);
+	} else {
+	    iPtr->legacyResult = "interpreter uses an incompatible stubs mechanism";
+	    iPtr->legacyFreeProc = 0; /* TCL_STATIC */
+	}
 	return NULL;
     }
 
     actualVersion = stubsPtr->tcl_PkgRequireEx(interp, tclName, version, 0, &pkgData);
     if (actualVersion == NULL) {
+	/* Even when the Tcl version does not match, the caller should at least be
+	 * able to use Tcl_GetObjResult/Tcl_GetString/Tcl_Panic for error-handling */
+	tclStubsPtr = stubsPtr; /* See: [fd8341e496] */
 	return NULL;
     }
     if (exact&1) {
