@@ -4,7 +4,8 @@
  *	This file implements Platform independent thread operations. Most of
  *	the real work is done in the platform dependent files.
  *
- * Copyright (c) 1998 by Sun Microsystems, Inc.
+ * Copyright © 1998 Sun Microsystems, Inc.
+ * Copyright © 2008 George Peter Staplin
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -39,21 +40,6 @@ static void		ForgetSyncObject(void *objPtr, SyncObjRecord *recPtr);
 static void		RememberSyncObject(void *objPtr,
 			    SyncObjRecord *recPtr);
 
-/*
- * Several functions are #defined to nothing in tcl.h if TCL_THREADS is not
- * specified. Here we undo that so the functions are defined in the stubs
- * table.
- */
-
-#ifndef TCL_THREADS
-#undef Tcl_MutexLock
-#undef Tcl_MutexUnlock
-#undef Tcl_MutexFinalize
-#undef Tcl_ConditionNotify
-#undef Tcl_ConditionWait
-#undef Tcl_ConditionFinalize
-#endif
-
 /*
  *----------------------------------------------------------------------
  *
@@ -75,28 +61,30 @@ static void		RememberSyncObject(void *objPtr,
 void *
 Tcl_GetThreadData(
     Tcl_ThreadDataKey *keyPtr,	/* Identifier for the data chunk */
-    int size)			/* Size of storage block */
+    Tcl_Size size)		/* Size of storage block */
 {
     void *result;
-#ifdef TCL_THREADS
+#if TCL_THREADS
     /*
      * Initialize the key for this thread.
      */
-    result = TclpThreadDataKeyGet(keyPtr);
+
+    result = TclThreadStorageKeyGet(keyPtr);
 
     if (result == NULL) {
-	result = ckalloc((size_t) size);
-	memset(result, 0, (size_t) size);
-	TclpThreadDataKeySet(keyPtr, result);
+	result = Tcl_Alloc(size);
+	memset(result, 0, size);
+	TclThreadStorageKeySet(keyPtr, result);
     }
 #else /* TCL_THREADS */
     if (*keyPtr == NULL) {
-	result = ckalloc((size_t) size);
-	memset(result, 0, (size_t) size);
+	result = Tcl_Alloc(size);
+	memset(result, 0, size);
 	*keyPtr = (Tcl_ThreadDataKey)result;
 	RememberSyncObject(keyPtr, &keyRecord);
+    } else {
+	result = *keyPtr;
     }
-    result = * (void **) keyPtr;
 #endif /* TCL_THREADS */
     return result;
 }
@@ -120,17 +108,15 @@ Tcl_GetThreadData(
 
 void *
 TclThreadDataKeyGet(
-    Tcl_ThreadDataKey *keyPtr)	/* Identifier for the data chunk, really
-				 * (pthread_key_t **) */
+    Tcl_ThreadDataKey *keyPtr)	/* Identifier for the data chunk. */
+
 {
-#ifdef TCL_THREADS
-    return TclpThreadDataKeyGet(keyPtr);
+#if TCL_THREADS
+    return TclThreadStorageKeyGet(keyPtr);
 #else /* TCL_THREADS */
-    char *result = *(char **) keyPtr;
-    return result;
+    return *keyPtr;
 #endif /* TCL_THREADS */
 }
-
 
 /*
  *----------------------------------------------------------------------
@@ -140,7 +126,7 @@ TclThreadDataKeyGet(
  *	Keep a list of (mutexes/condition variable/data key) used during
  *	finalization.
  *
- *	Assume master lock is held.
+ *	Assume global lock is held.
  *
  * Results:
  *	None.
@@ -159,7 +145,6 @@ RememberSyncObject(
     void **newList;
     int i, j;
 
-
     /*
      * Reuse any free slot in the list.
      */
@@ -178,14 +163,14 @@ RememberSyncObject(
 
     if (recPtr->num >= recPtr->max) {
 	recPtr->max += 8;
-	newList = (void **) ckalloc(recPtr->max * sizeof(char *));
+	newList = (void **)Tcl_Alloc(recPtr->max * sizeof(void *));
 	for (i=0,j=0 ; i<recPtr->num ; i++) {
 	    if (recPtr->list[i] != NULL) {
 		newList[j++] = recPtr->list[i];
 	    }
 	}
 	if (recPtr->list != NULL) {
-	    ckfree((char *) recPtr->list);
+	    Tcl_Free(recPtr->list);
 	}
 	recPtr->list = newList;
 	recPtr->num = j;
@@ -201,7 +186,7 @@ RememberSyncObject(
  * ForgetSyncObject
  *
  *	Remove a single object from the list.
- *	Assume master lock is held.
+ *	Assume global lock is held.
  *
  * Results:
  *	None.
@@ -233,7 +218,7 @@ ForgetSyncObject(
  * TclRememberMutex
  *
  *	Keep a list of mutexes used during finalization.
- *	Assume master lock is held.
+ *	Assume global lock is held.
  *
  * Results:
  *	None.
@@ -268,16 +253,17 @@ TclRememberMutex(
  *----------------------------------------------------------------------
  */
 
+#undef Tcl_MutexFinalize
 void
 Tcl_MutexFinalize(
     Tcl_Mutex *mutexPtr)
 {
-#ifdef TCL_THREADS
+#if TCL_THREADS
     TclpFinalizeMutex(mutexPtr);
 #endif
-    TclpMasterLock();
+    TclpGlobalLock();
     ForgetSyncObject(mutexPtr, &mutexRecord);
-    TclpMasterUnlock();
+    TclpGlobalUnlock();
 }
 
 /*
@@ -286,7 +272,7 @@ Tcl_MutexFinalize(
  * TclRememberCondition
  *
  *	Keep a list of condition variables used during finalization.
- *	Assume master lock is held.
+ *	Assume global lock is held.
  *
  * Results:
  *	None.
@@ -321,16 +307,17 @@ TclRememberCondition(
  *----------------------------------------------------------------------
  */
 
+#undef Tcl_ConditionFinalize
 void
 Tcl_ConditionFinalize(
     Tcl_Condition *condPtr)
 {
-#ifdef TCL_THREADS
+#if TCL_THREADS
     TclpFinalizeCondition(condPtr);
 #endif
-    TclpMasterLock();
+    TclpGlobalLock();
     ForgetSyncObject(condPtr, &condRecord);
-    TclpMasterUnlock();
+    TclpGlobalUnlock();
 }
 
 /*
@@ -352,11 +339,18 @@ Tcl_ConditionFinalize(
  */
 
 void
-TclFinalizeThreadData(void)
+TclFinalizeThreadData(int quick)
 {
-    TclpFinalizeThreadDataThread();
-#if defined(TCL_THREADS) && defined(USE_THREAD_ALLOC)
-    TclFinalizeThreadAllocThread();
+    TclFinalizeThreadDataThread();
+#if TCL_THREADS && defined(USE_THREAD_ALLOC)
+    if (!quick) {
+	/*
+	 * Quick exit principle makes it useless to terminate allocators
+	 */
+	TclFinalizeThreadAllocThread();
+    }
+#else
+    (void)quick;
 #endif
 }
 
@@ -383,11 +377,11 @@ TclFinalizeSynchronization(void)
     int i;
     void *blockPtr;
     Tcl_ThreadDataKey *keyPtr;
-#ifdef TCL_THREADS
+#if TCL_THREADS
     Tcl_Mutex *mutexPtr;
     Tcl_Condition *condPtr;
 
-    TclpMasterLock();
+    TclpGlobalLock();
 #endif
 
     /*
@@ -398,18 +392,18 @@ TclFinalizeSynchronization(void)
     if (keyRecord.list != NULL) {
 	for (i=0 ; i<keyRecord.num ; i++) {
 	    keyPtr = (Tcl_ThreadDataKey *) keyRecord.list[i];
-	    blockPtr = (void *) *keyPtr;
-	    ckfree(blockPtr);
+	    blockPtr = *keyPtr;
+	    Tcl_Free(blockPtr);
 	}
-	ckfree((char *) keyRecord.list);
+	Tcl_Free(keyRecord.list);
 	keyRecord.list = NULL;
     }
     keyRecord.max = 0;
     keyRecord.num = 0;
-    
-#ifdef TCL_THREADS
+
+#if TCL_THREADS
     /*
-     * Call thread storage master cleanup.
+     * Call thread storage global cleanup.
      */
 
     TclFinalizeThreadStorage();
@@ -421,7 +415,7 @@ TclFinalizeSynchronization(void)
 	}
     }
     if (mutexRecord.list != NULL) {
-	ckfree((char *) mutexRecord.list);
+	Tcl_Free(mutexRecord.list);
 	mutexRecord.list = NULL;
     }
     mutexRecord.max = 0;
@@ -434,13 +428,13 @@ TclFinalizeSynchronization(void)
 	}
     }
     if (condRecord.list != NULL) {
-	ckfree((char *) condRecord.list);
+	Tcl_Free(condRecord.list);
 	condRecord.list = NULL;
     }
     condRecord.max = 0;
     condRecord.num = 0;
 
-    TclpMasterUnlock();
+    TclpGlobalUnlock();
 #endif /* TCL_THREADS */
 }
 
@@ -462,17 +456,15 @@ TclFinalizeSynchronization(void)
  *----------------------------------------------------------------------
  */
 
-void
+TCL_NORETURN void
 Tcl_ExitThread(
     int status)
 {
     Tcl_FinalizeThread();
-#ifdef TCL_THREADS
     TclpThreadExit(status);
-#endif
 }
 
-#ifndef TCL_THREADS
+#if !TCL_THREADS
 
 /*
  *----------------------------------------------------------------------
@@ -495,30 +487,30 @@ Tcl_ExitThread(
 #undef Tcl_ConditionWait
 void
 Tcl_ConditionWait(
-    Tcl_Condition *condPtr,	/* Really (pthread_cond_t **) */
-    Tcl_Mutex *mutexPtr,	/* Really (pthread_mutex_t **) */
-    Tcl_Time *timePtr)		/* Timeout on waiting period */
+    TCL_UNUSED(Tcl_Condition *),	/* Really (pthread_cond_t **) */
+    TCL_UNUSED(Tcl_Mutex *),	/* Really (pthread_mutex_t **) */
+    TCL_UNUSED(const Tcl_Time *)) /* Timeout on waiting period */
 {
 }
 
 #undef Tcl_ConditionNotify
 void
 Tcl_ConditionNotify(
-    Tcl_Condition *condPtr)
+    TCL_UNUSED(Tcl_Condition *))
 {
 }
 
 #undef Tcl_MutexLock
 void
 Tcl_MutexLock(
-    Tcl_Mutex *mutexPtr)
+    TCL_UNUSED(Tcl_Mutex *))
 {
 }
 
 #undef Tcl_MutexUnlock
 void
 Tcl_MutexUnlock(
-    Tcl_Mutex *mutexPtr)
+    TCL_UNUSED(Tcl_Mutex *))
 {
 }
 #endif /* !TCL_THREADS */
