@@ -94,6 +94,8 @@ typedef	struct finderinfo {
 } __attribute__ ((__packed__)) finderinfo;
 
 typedef struct {
+    u_int64_t reserved1; /* Make sure data is 8-byte aligned */
+    u_int32_t reserved2; /* See [992f94d847] */
     u_int32_t info_length;
     u_int32_t data[8];
 } fileinfobuf;
@@ -160,7 +162,8 @@ TclMacOSXGetFileAttribute(
 	alist.commonattr = ATTR_CMN_FNDRINFO;
     }
     native = (const char *)Tcl_FSGetNativePath(fileName);
-    result = getattrlist(native, &alist, &finfo, sizeof(fileinfobuf), 0);
+    result = getattrlist(native, &alist, &finfo.info_length,
+	    sizeof(fileinfobuf) - offsetof(fileinfobuf, info_length), 0);
 
     if (result != 0) {
 	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
@@ -256,7 +259,8 @@ TclMacOSXSetFileAttribute(
 	alist.commonattr = ATTR_CMN_FNDRINFO;
     }
     native = (const char *)Tcl_FSGetNativePath(fileName);
-    result = getattrlist(native, &alist, &finfo, sizeof(fileinfobuf), 0);
+    result = getattrlist(native, &alist, &finfo.info_length,
+	    sizeof(fileinfobuf) - offsetof(fileinfobuf, info_length), 0);
 
     if (result != 0) {
 	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
@@ -413,7 +417,8 @@ TclMacOSXCopyFileAttributes(
 	alist.bitmapcount = ATTR_BIT_MAP_COUNT;
 	alist.commonattr = ATTR_CMN_FNDRINFO;
 
-	if (getattrlist(src, &alist, &finfo, sizeof(fileinfobuf), 0)) {
+	if (getattrlist(src, &alist, &finfo.info_length,
+		sizeof(fileinfobuf) - offsetof(fileinfobuf, info_length), 0)) {
 	    return TCL_ERROR;
 	}
 	if (setattrlist(dst, &alist, &finfo.data, sizeof(finfo.data), 0)) {
@@ -435,7 +440,8 @@ TclMacOSXCopyFileAttributes(
 
 	alist.commonattr = 0;
 	alist.fileattr = ATTR_FILE_RSRCLENGTH;
-	if (getattrlist(src, &alist, &finfo, sizeof(fileinfobuf), 0)) {
+	if (getattrlist(src, &alist, &finfo.info_length,
+		sizeof(fileinfobuf) - offsetof(fileinfobuf, info_length), 0)) {
 	    return TCL_ERROR;
 	} else if (*rsrcForkSize == 0) {
 	    return TCL_OK;
@@ -504,7 +510,8 @@ TclMacOSXMatchType(
     bzero(&alist, sizeof(struct attrlist));
     alist.bitmapcount = ATTR_BIT_MAP_COUNT;
     alist.commonattr = ATTR_CMN_FNDRINFO;
-    if (getattrlist(pathName, &alist, &finfo, sizeof(fileinfobuf), 0) != 0) {
+    if (getattrlist(pathName, &alist, &finfo.info_length,
+	    sizeof(fileinfobuf) - offsetof(fileinfobuf, info_length), 0)) {
 	return 0;
     }
     if ((types->perm & TCL_GLOB_PERM_HIDDEN) &&
@@ -687,7 +694,7 @@ UpdateStringOfOSType(
     Tcl_Encoding encoding;
     char src[5];
 
-    TclOOM(dst, size);
+    TclOOM(dst, size+1);
 
     src[0] = (char) (osType >> 24);
     src[1] = (char) (osType >> 16);
