@@ -4,7 +4,7 @@
  *	This file contains a simple Tcl package "pkga" that is intended for
  *	testing the Tcl dynamic loading facilities.
  *
- * Copyright (c) 1995 Sun Microsystems, Inc.
+ * Copyright © 1995 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -13,15 +13,6 @@
 #undef STATIC_BUILD
 #include "tcl.h"
 
-/*
- * Prototypes for procedures defined later in this file:
- */
-
-static int    Pkga_EqObjCmd(ClientData clientData,
-		Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]);
-static int    Pkga_QuoteObjCmd(ClientData clientData,
-		Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]);
-
 /*
  *----------------------------------------------------------------------
  *
@@ -42,14 +33,14 @@ static int    Pkga_QuoteObjCmd(ClientData clientData,
 
 static int
 Pkga_EqObjCmd(
-    ClientData dummy,		/* Not used. */
+    void *dummy,		/* Not used. */
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument objects. */
 {
     int result;
     const char *str1, *str2;
-    int len1, len2;
+    Tcl_Size len1, len2;
     (void)dummy;
 
     if (objc != 3) {
@@ -59,8 +50,10 @@ Pkga_EqObjCmd(
 
     str1 = Tcl_GetStringFromObj(objv[1], &len1);
     str2 = Tcl_GetStringFromObj(objv[2], &len2);
+    len1 = Tcl_NumUtfChars(str1, len1);
+    len2 = Tcl_NumUtfChars(str2, len2);
     if (len1 == len2) {
-	result = (Tcl_UtfNcmp(str1, str2, len1) == 0);
+	result = (Tcl_UtfNcmp(str1, str2, (size_t)len1) == 0);
     } else {
 	result = 0;
     }
@@ -87,7 +80,7 @@ Pkga_EqObjCmd(
 
 static int
 Pkga_QuoteObjCmd(
-    ClientData dummy,		/* Not used. */
+    void *dummy,		/* Not used. */
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument strings. */
@@ -126,7 +119,7 @@ Pkga_Init(
 {
     int code;
 
-    if (Tcl_InitStubs(interp, "8.5", 0) == NULL) {
+    if (Tcl_InitStubs(interp, "8.5-", 0) == NULL) {
 	return TCL_ERROR;
     }
     code = Tcl_PkgProvide(interp, "pkga", "1.0");
