@@ -5,7 +5,7 @@
  *	testing the Tcl dynamic loading facilities. Its Init procedure returns
  *	an error in order to test how this is handled.
  *
- * Copyright (c) 1995 Sun Microsystems, Inc.
+ * Copyright © 1995 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -41,5 +41,5 @@ Pkge_Init(
     if (Tcl_InitStubs(interp, "8.5-", 0) == NULL) {
 	return TCL_ERROR;
     }
-    return Tcl_EvalEx(interp, script, -1, 0);
+    return Tcl_EvalEx(interp, script, TCL_INDEX_NONE, 0);
 }
