@@ -254,36 +254,33 @@ typedef struct {
     const void *lpszClassName;
 } WNDCLASSW;
 
-#ifdef __clang__
-#pragma clang diagnostic ignored "-Wignored-attributes"
-#endif
-extern void __stdcall	CloseHandle(void *);
-extern void *__stdcall	CreateEventW(void *, unsigned char, unsigned char,
+extern void CloseHandle(void *);
+extern void *CreateEventW(void *, unsigned char, unsigned char,
 			    void *);
-extern void *__stdcall	CreateWindowExW(void *, const void *, const void *,
+extern void *CreateWindowExW(void *, const void *, const void *,
 			    unsigned int, int, int, int, int, void *, void *,
 			    void *, void *);
-extern unsigned int __stdcall	DefWindowProcW(void *, int, void *, void *);
-extern unsigned char __stdcall	DestroyWindow(void *);
-extern int __stdcall	DispatchMessageW(const MSG *);
-extern unsigned char __stdcall	GetMessageW(MSG *, void *, int, int);
-extern void __stdcall	MsgWaitForMultipleObjects(unsigned int, void *,
+extern unsigned int DefWindowProcW(void *, int, void *, void *);
+extern unsigned char DestroyWindow(void *);
+extern int DispatchMessageW(const MSG *);
+extern unsigned char GetMessageW(MSG *, void *, int, int);
+extern void MsgWaitForMultipleObjects(unsigned int, void *,
 			    unsigned char, unsigned int, unsigned int);
-extern unsigned char __stdcall	PeekMessageW(MSG *, void *, int, int, int);
-extern unsigned char __stdcall	PostMessageW(void *, unsigned int, void *,
+extern unsigned char PeekMessageW(MSG *, void *, int, int, int);
+extern unsigned char PostMessageW(void *, unsigned int, void *,
 				    void *);
-extern void __stdcall	PostQuitMessage(int);
-extern void *__stdcall	RegisterClassW(const WNDCLASSW *);
-extern unsigned char __stdcall	ResetEvent(void *);
-extern unsigned char __stdcall	TranslateMessage(const MSG *);
+extern void PostQuitMessage(int);
+extern void *RegisterClassW(const WNDCLASSW *);
+extern unsigned char ResetEvent(void *);
+extern unsigned char TranslateMessage(const MSG *);
 
 /*
  * Threaded-cygwin specific constants and functions in this file:
  */
 
 #if TCL_THREADS && defined(__CYGWIN__)
-static const wchar_t className[] = L"TclNotifier";
-static unsigned int __stdcall	NotifierProc(void *hwnd, unsigned int message,
+static const WCHAR className[] = L"TclNotifier";
+static unsigned int NotifierProc(void *hwnd, unsigned int message,
 			    void *wParam, void *lParam);
 #endif /* TCL_THREADS && defined(__CYGWIN__) */
 #ifdef __cplusplus
@@ -600,7 +597,7 @@ TclpDeleteFileHandler(
 
 #if TCL_THREADS && defined(__CYGWIN__)
 
-static unsigned int __stdcall
+static unsigned int
 NotifierProc(
     void *hwnd,
     unsigned int message,
