@@ -12,6 +12,9 @@
 
 #include "tclInt.h"
 #include "tclFileSystem.h"
+#if !defined(NO_DLFCN_H)
+#include <dlfcn.h>
+#endif
 
 static int		NativeMatchType(Tcl_Interp *interp,
 			    const char* nativeEntry, const char* nativeName,
@@ -26,23 +29,25 @@ static int		NativeMatchType(Tcl_Interp *interp,
  *	application, given its argv[0] value. For Cygwin, argv[0] is
  *	ignored and the path is determined the same as under win32.
  *
+ *	In the case of shared Tcl library, the absolute path name of the
+ *	Tcl library is also determined.
+ *
  * Results:
  *	None.
  *
  * Side effects:
- *	The computed path name is stored as a ProcessGlobalValue.
+ *	The computed path name(s) are stored as ProcessGlobalValue entries.
  *
  *---------------------------------------------------------------------------
  */
-
 #ifdef __CYGWIN__
 void
 TclpFindExecutable(
     TCL_UNUSED(const char *) /*argv0*/)
 {
     size_t length;
-    wchar_t buf[PATH_MAX] = L"";
-    char name[PATH_MAX * 3 + 1];
+    WCHAR buf[PATH_MAX] = L"";
+    char name[PATH_MAX * TCL_UTF_MAX + 1];
 
     GetModuleFileNameW(NULL, buf, PATH_MAX);
     cygwin_conv_path(3, buf, name, sizeof(name));
@@ -51,8 +56,15 @@ TclpFindExecutable(
 	/* Strip '.exe' part. */
 	length -= 4;
     }
-    TclSetObjNameOfExecutable(
-	    Tcl_NewStringObj(name, length), NULL);
+    TclSetObjNameOfExecutable(Tcl_NewStringObj(name, length), NULL);
+
+#if !defined(STATIC_BUILD)
+    HMODULE hModule = (HMODULE)TclWinGetTclInstance();
+    if (GetModuleFileNameW(hModule, buf, PATH_MAX) < PATH_MAX) {
+	cygwin_conv_path(3, buf, name, sizeof(name));
+    }
+    TclSetObjNameOfShlib(Tcl_NewStringObj(name, TCL_AUTO_LENGTH), NULL);
+#endif
 }
 #else
 void
@@ -141,7 +153,7 @@ TclpFindExecutable(
     }
     TclNewObj(obj);
     TclSetObjNameOfExecutable(obj, NULL);
-    goto done;
+    goto getShlibName;
 
     /*
      * If the name starts with "/" then just store it
@@ -156,13 +168,13 @@ TclpFindExecutable(
     {
 	Tcl_ExternalToUtfDStringEx(NULL, NULL, name, TCL_INDEX_NONE, TCL_ENCODING_PROFILE_TCL8, &utfName, NULL);
 	TclSetObjNameOfExecutable(Tcl_DStringToObj(&utfName), NULL);
-	goto done;
+	goto getShlibName;
     }
 
     if (TclpGetCwd(NULL, &cwd) == NULL) {
 	TclNewObj(obj);
 	TclSetObjNameOfExecutable(obj, NULL);
-	goto done;
+	goto getShlibName;
     }
 
     /*
@@ -192,7 +204,18 @@ TclpFindExecutable(
 	    TCL_ENCODING_PROFILE_TCL8, &utfName, NULL);
     TclSetObjNameOfExecutable(Tcl_DStringToObj(&utfName), NULL);
 
-  done:
+  getShlibName:
+#if !defined(STATIC_BUILD)
+    name = CFG_RUNTIME_LIBDIR "/" CFG_RUNTIME_DLLFILE;
+# if !defined(NO_DLFCN_H)
+    Dl_info dlinfo;
+    if (dladdr((const void *)TclpFindExecutable, &dlinfo) && dlinfo.dli_fname) {
+	name = dlinfo.dli_fname;
+    }
+# endif
+    TclSetObjNameOfShlib(Tcl_NewStringObj(name, TCL_AUTO_LENGTH), NULL);
+#endif /* STATIC_BUILD */
+
     Tcl_DStringFree(&buffer);
 }
 #endif
@@ -317,7 +340,7 @@ TclpMatchInDirectory(
 	    return TCL_OK;
 	}
 
-	d = TclOSopendir(native);				/* INTL: Native. */
+	d = TclOSopendir(native);			/* INTL: Native. */
 	if (d == NULL) {
 	    Tcl_DStringFree(&ds);
 	    if (interp != NULL) {
@@ -431,10 +454,10 @@ TclpMatchInDirectory(
 
 static int
 NativeMatchType(
-    Tcl_Interp *interp,       /* Interpreter to receive errors. */
-    const char *nativeEntry,  /* Native path to check. */
-    const char *nativeName,   /* Native filename to check. */
-    Tcl_GlobTypeData *types)  /* Type description to match against. */
+    Tcl_Interp *interp,		/* Interpreter to receive errors. */
+    const char *nativeEntry,	/* Native path to check. */
+    const char *nativeName,	/* Native filename to check. */
+    Tcl_GlobTypeData *types)	/* Type description to match against. */
 {
     Tcl_StatBuf buf;
 
@@ -1237,13 +1260,13 @@ TclOSfstat(
     Tcl_StatBuf *statBuf = (Tcl_StatBuf *)cygstat;
     int result = fstat(fd, &buf);
 
-    statBuf->st_mode = buf.st_mode;
-    statBuf->st_ino = buf.st_ino;
+    statBuf->st_mode = (unsigned short)buf.st_mode;
+    statBuf->st_ino = (unsigned short)buf.st_ino;
     statBuf->st_dev = buf.st_dev;
     statBuf->st_rdev = buf.st_rdev;
     statBuf->st_nlink = buf.st_nlink;
-    statBuf->st_uid = buf.st_uid;
-    statBuf->st_gid = buf.st_gid;
+    statBuf->st_uid = (short)buf.st_uid;
+    statBuf->st_gid = (short)buf.st_gid;
     statBuf->st_size = buf.st_size;
     statBuf->st_atime = buf.st_atime;
     statBuf->st_mtime = buf.st_mtime;
@@ -1260,13 +1283,13 @@ TclOSstat(
     Tcl_StatBuf *statBuf = (Tcl_StatBuf *)cygstat;
     int result = stat(name, &buf);
 
-    statBuf->st_mode = buf.st_mode;
-    statBuf->st_ino = buf.st_ino;
+    statBuf->st_mode = (unsigned short)buf.st_mode;
+    statBuf->st_ino = (unsigned short)buf.st_ino;
     statBuf->st_dev = buf.st_dev;
     statBuf->st_rdev = buf.st_rdev;
     statBuf->st_nlink = buf.st_nlink;
-    statBuf->st_uid = buf.st_uid;
-    statBuf->st_gid = buf.st_gid;
+    statBuf->st_uid = (short)buf.st_uid;
+    statBuf->st_gid = (short)buf.st_gid;
     statBuf->st_size = buf.st_size;
     statBuf->st_atime = buf.st_atime;
     statBuf->st_mtime = buf.st_mtime;
@@ -1283,13 +1306,13 @@ TclOSlstat(
     Tcl_StatBuf *statBuf = (Tcl_StatBuf *)cygstat;
     int result = lstat(name, &buf);
 
-    statBuf->st_mode = buf.st_mode;
-    statBuf->st_ino = buf.st_ino;
+    statBuf->st_mode = (unsigned short)buf.st_mode;
+    statBuf->st_ino = (unsigned short)buf.st_ino;
     statBuf->st_dev = buf.st_dev;
     statBuf->st_rdev = buf.st_rdev;
     statBuf->st_nlink = buf.st_nlink;
-    statBuf->st_uid = buf.st_uid;
-    statBuf->st_gid = buf.st_gid;
+    statBuf->st_uid = (short)buf.st_uid;
+    statBuf->st_gid = (short)buf.st_gid;
     statBuf->st_size = buf.st_size;
     statBuf->st_atime = buf.st_atime;
     statBuf->st_mtime = buf.st_mtime;
