@@ -89,25 +89,18 @@ extern "C" {
 #   define SOCKET unsigned int
 #   define WSAEWOULDBLOCK 10035
     typedef unsigned short WCHAR;
-#ifdef __clang__
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wignored-attributes"
-#endif
-    __declspec(dllimport) extern __stdcall int GetModuleHandleExW(unsigned int, const void *, void *);
-    __declspec(dllimport) extern __stdcall int GetModuleFileNameW(void *, const void *, int);
-    __declspec(dllimport) extern __stdcall int WideCharToMultiByte(int, int, const void *, int,
+    __declspec(dllimport) extern int GetModuleHandleExW(unsigned int, const void *, void *);
+    __declspec(dllimport) extern int GetModuleFileNameW(void *, const void *, int);
+    __declspec(dllimport) extern int WideCharToMultiByte(int, int, const void *, int,
 	    char *, int, const char *, void *);
-    __declspec(dllimport) extern __stdcall int MultiByteToWideChar(int, int, const char *, int,
+    __declspec(dllimport) extern int MultiByteToWideChar(int, int, const char *, int,
 	    WCHAR *, int);
-    __declspec(dllimport) extern __stdcall void OutputDebugStringW(const WCHAR *);
-    __declspec(dllimport) extern __stdcall int IsDebuggerPresent(void);
-    __declspec(dllimport) extern __stdcall int GetLastError(void);
-    __declspec(dllimport) extern __stdcall int GetFileAttributesW(const WCHAR *);
-    __declspec(dllimport) extern __stdcall int SetFileAttributesW(const WCHAR *, int);
+    __declspec(dllimport) extern void OutputDebugStringW(const WCHAR *);
+    __declspec(dllimport) extern int IsDebuggerPresent(void);
+    __declspec(dllimport) extern int GetLastError(void);
+    __declspec(dllimport) extern int GetFileAttributesW(const WCHAR *);
+    __declspec(dllimport) extern int SetFileAttributesW(const WCHAR *, int);
     __declspec(dllimport) extern int cygwin_conv_path(int, const void *, void *, int);
-#ifdef __clang__
-#pragma clang diagnostic pop
-#endif
 #   define timezone _timezone
     extern int TclOSfstat(int fd, void *statBuf);
     extern int TclOSstat(const char *name, void *statBuf);
@@ -559,9 +552,7 @@ extern char **		environ;
  *---------------------------------------------------------------------------
  */
 
-#   ifdef HAVE_AVAILABILITYMACROS_H
-#	include <AvailabilityMacros.h>
-#   endif
+#    include <AvailabilityMacros.h>
 
 /*
  *---------------------------------------------------------------------------
@@ -570,41 +561,11 @@ extern char **		environ;
  */
 
 #   ifdef HAVE_WEAK_IMPORT
-#	if !defined(HAVE_AVAILABILITYMACROS_H) || !defined(MAC_OS_X_VERSION_MIN_REQUIRED)
-#	    undef HAVE_WEAK_IMPORT
-#	else
-#	    ifndef WEAK_IMPORT_ATTRIBUTE
-#		define WEAK_IMPORT_ATTRIBUTE	__attribute__((weak_import))
-#	    endif
+#	ifndef WEAK_IMPORT_ATTRIBUTE
+#	    define WEAK_IMPORT_ATTRIBUTE	__attribute__((weak_import))
 #	endif
 #   endif /* HAVE_WEAK_IMPORT */
 
-/*
- *---------------------------------------------------------------------------
- * Support for MAC_OS_X_VERSION_MAX_ALLOWED define from AvailabilityMacros.h:
- * only use API available in the indicated OS version or earlier.
- *---------------------------------------------------------------------------
- */
-
-#   ifdef MAC_OS_X_VERSION_MAX_ALLOWED
-#	if MAC_OS_X_VERSION_MAX_ALLOWED < 1050 && defined(__LP64__)
-#	    undef HAVE_COREFOUNDATION
-#	endif
-#	if MAC_OS_X_VERSION_MAX_ALLOWED < 1040
-#	    undef HAVE_OSSPINLOCKLOCK
-#	    undef HAVE_PTHREAD_ATFORK
-#	    undef HAVE_COPYFILE
-#	endif
-#	if MAC_OS_X_VERSION_MAX_ALLOWED < 1030
-	    /* prior to 10.3, realpath is not threadsafe, c.f. bug 711232 */
-#	    define NO_REALPATH 1
-#	    undef HAVE_LANGINFO
-#	endif
-#   endif /* MAC_OS_X_VERSION_MAX_ALLOWED */
-#   if defined(HAVE_COREFOUNDATION) && defined(__LP64__) && \
-	    defined(HAVE_WEAK_IMPORT) && MAC_OS_X_VERSION_MIN_REQUIRED < 1050
-#	warning "Weak import of 64-bit CoreFoundation is not supported, will not run on Mac OS X < 10.5."
-#   endif
     /*
      * For now, test exec-17.1 fails (I/O setup after closing stdout) with
      * posix_spawnp(), but the classic implementation (based on fork()+execvp())
