@@ -61,55 +61,27 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef HAVE_DECL_PTHREAD_MUTEX_RECURSIVE
-#define HAVE_DECL_PTHREAD_MUTEX_RECURSIVE 0
+/*
+ * No correct native support for reentrant mutexes. Emulate them with a counter.
+ */
+
+#ifndef PTHREAD_NULL
+#   define PTHREAD_NULL (pthread_t)0
 #endif
 
-#if HAVE_DECL_PTHREAD_MUTEX_RECURSIVE
-/*
- * Pthread has native reentrant (AKA recursive) mutexes. Use them for
- * Tcl_Mutex.
- */
-
-typedef pthread_mutex_t PMutex;
-
-static void
-PMutexInit(
-    PMutex *pmutexPtr)
-{
-    pthread_mutexattr_t attr;
-
-    pthread_mutexattr_init(&attr);
-    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
-    pthread_mutex_init(pmutexPtr, &attr);
-}
-
-#define PMutexDestroy	pthread_mutex_destroy
-#define PMutexLock	pthread_mutex_lock
-#define PMutexUnlock	pthread_mutex_unlock
-#define PCondWait	pthread_cond_wait
-#define PCondTimedWait	pthread_cond_timedwait
-
-#else /* !HAVE_PTHREAD_MUTEX_RECURSIVE */
-
-/*
- * No native support for reentrant mutexes. Emulate them with regular mutexes
- * and thread-local counters.
- */
-
 typedef struct PMutex {
     pthread_mutex_t mutex;
-    pthread_t thread;
-    int counter;
+    volatile pthread_t thread;
+    int counter; // Number of additional locks in the same thread.
 } PMutex;
 
 static void
 PMutexInit(
     PMutex *pmutexPtr)
 {
-    pthread_mutex_init(&pmutexPtr->mutex, NULL);
-    pmutexPtr->thread = 0;
+    pmutexPtr->thread = PTHREAD_NULL;
     pmutexPtr->counter = 0;
+    pthread_mutex_init(&pmutexPtr->mutex, NULL);
 }
 
 static void
@@ -117,37 +89,54 @@ PMutexDestroy(
     PMutex *pmutexPtr)
 {
     pthread_mutex_destroy(&pmutexPtr->mutex);
+    assert(pthread_equal(pmutexPtr->thread, PTHREAD_NULL) && !pmutexPtr->counter);
 }
 
 static void
 PMutexLock(
     PMutex *pmutexPtr)
 {
-    if (pmutexPtr->thread != pthread_self() || pmutexPtr->counter == 0) {
+    pthread_t mythread = pthread_self();
+
+    if (pthread_equal(pmutexPtr->thread, mythread)) {
+	// We own the lock already, so it's recursive.
+	pmutexPtr->counter++;
+    } else {
+	// We don't owns the lock, so we have to lock it. Then we own it.
 	pthread_mutex_lock(&pmutexPtr->mutex);
-	pmutexPtr->thread = pthread_self();
-	pmutexPtr->counter = 0;
+	pmutexPtr->thread = mythread;
     }
-    pmutexPtr->counter++;
 }
 
 static void
 PMutexUnlock(
     PMutex *pmutexPtr)
 {
-    pmutexPtr->counter--;
-    if (pmutexPtr->counter == 0) {
-	pmutexPtr->thread = 0;
+    assert(pthread_equal(pmutexPtr->thread, pthread_self()));
+    if (pmutexPtr->counter) {
+	// It's recursive
+	pmutexPtr->counter--;
+    } else {
+	pmutexPtr->thread = PTHREAD_NULL;
 	pthread_mutex_unlock(&pmutexPtr->mutex);
     }
 }
 
+
 static void
 PCondWait(
     pthread_cond_t *pcondPtr,
     PMutex *pmutexPtr)
 {
+    pthread_t mythread = pthread_self();
+
+    assert(pthread_equal(pmutexPtr->thread, mythread));
+    int counter = pmutexPtr->counter;
+    pmutexPtr->counter = 0;
+    pmutexPtr->thread = PTHREAD_NULL;
     pthread_cond_wait(pcondPtr, &pmutexPtr->mutex);
+    pmutexPtr->thread = mythread;
+    pmutexPtr->counter = counter;
 }
 
 static void
@@ -156,10 +145,17 @@ PCondTimedWait(
     PMutex *pmutexPtr,
     struct timespec *ptime)
 {
+    pthread_t mythread = pthread_self();
+
+    assert(pthread_equal(pmutexPtr->thread, mythread));
+    int counter = pmutexPtr->counter;
+    pmutexPtr->counter = 0;
+    pmutexPtr->thread = PTHREAD_NULL;
     pthread_cond_timedwait(pcondPtr, &pmutexPtr->mutex, ptime);
+    pmutexPtr->thread = mythread;
+    pmutexPtr->counter = counter;
 }
-#endif /* HAVE_PTHREAD_MUTEX_RECURSIVE */
-
+
 /*
  * globalLock is used to serialize creation of mutexes, condition variables,
  * and thread local storage. This is the only place that can count on the
@@ -213,8 +209,8 @@ int
 TclpThreadCreate(
     Tcl_ThreadId *idPtr,	/* Return, the ID of the thread */
     Tcl_ThreadCreateProc *proc,	/* Main() function of the thread */
-    void *clientData,	/* The one argument to Main() */
-    size_t stackSize,	/* Size of stack for the new thread */
+    void *clientData,		/* The one argument to Main() */
+    size_t stackSize,		/* Size of stack for the new thread */
     int flags)			/* Flags controlling behaviour of the new
 				 * thread. */
 {
@@ -532,10 +528,8 @@ Tcl_Mutex *
 Tcl_GetAllocMutex(void)
 {
 #if TCL_THREADS
-    PMutex **allocLockPtrPtr = &allocLockPtr;
-
     pthread_once(&allocLockInitOnce, allocLockInit);
-    return (Tcl_Mutex *) allocLockPtrPtr;
+    return (Tcl_Mutex *) &allocLockPtr;
 #else
     return NULL;
 #endif
@@ -637,7 +631,7 @@ void
 TclpFinalizeMutex(
     Tcl_Mutex *mutexPtr)
 {
-    PMutex *pmutexPtr = *(PMutex **) mutexPtr;
+    PMutex *pmutexPtr = *(PMutex **)mutexPtr;
 
     if (pmutexPtr != NULL) {
 	PMutexDestroy(pmutexPtr);
@@ -672,7 +666,7 @@ void
 Tcl_ConditionWait(
     Tcl_Condition *condPtr,	/* Really (pthread_cond_t **) */
     Tcl_Mutex *mutexPtr,	/* Really (PMutex **) */
-    const Tcl_Time *timePtr) /* Timeout on waiting period */
+    const Tcl_Time *timePtr)	/* Timeout on waiting period */
 {
     pthread_cond_t *pcondPtr;
     PMutex *pmutexPtr;
