@@ -12,10 +12,12 @@
  */
 
 #include "tclWinInt.h"
-#include <assert.h>
 #include <winnt.h>
 #include <winbase.h>
 #include <lmcons.h>
+#if defined (__clang__) && (__clang_major__ > 20)
+#pragma clang diagnostic ignored "-Wc++-keyword"
+#endif
 
 /*
  * GetUserNameW() is found in advapi32.dll
@@ -331,6 +333,7 @@ AppendEnvironment(
 {
     Tcl_Size pathc;
     WCHAR wBuf[MAX_PATH];
+    DWORD dw;
     char buf[MAX_PATH * 3];
     Tcl_Obj *objPtr;
     Tcl_DString ds;
@@ -355,13 +358,14 @@ AppendEnvironment(
 	Tcl_Panic("no '/' character found in lib");
     }
 
-    /*
-     * The "L" preceding the TCL_LIBRARY string is used to tell VC++ that
-     * this is a Unicode string.
-     */
-
-    GetEnvironmentVariableW(L"TCL_LIBRARY", wBuf, MAX_PATH);
-    WideCharToMultiByte(CP_UTF8, 0, wBuf, -1, buf, MAX_PATH * 3, NULL, NULL);
+    dw = GetEnvironmentVariableW(L"TCL_LIBRARY", wBuf, MAX_PATH);
+    if (dw <= 0 || dw >= MAX_PATH) {
+	return;
+    }
+    if (WideCharToMultiByte(
+	    CP_UTF8, 0, wBuf, -1, buf, MAX_PATH * 3, NULL, NULL) == 0) {
+	return;
+    }
 
     if (buf[0] != '\0') {
 	objPtr = Tcl_NewStringObj(buf, TCL_INDEX_NONE);
@@ -525,7 +529,8 @@ TclpSetInitialEncodings(void)
 }
 
 const char *
-Tcl_GetEncodingNameForUser(Tcl_DString *bufPtr)
+Tcl_GetEncodingNameForUser(
+    Tcl_DString *bufPtr)
 {
     Tcl_DStringInit(bufPtr);
     Tcl_DStringAppend(bufPtr, TclpGetCodePage(), -1);
