@@ -13,6 +13,10 @@
  */
 
 #include "tclWinInt.h"
+#if defined (__clang__) && (__clang_major__ > 20)
+#pragma clang diagnostic ignored "-Wc++-keyword"
+#endif
+
 
 /*
  * The following variable is used to tell whether this module has been
