@@ -43,10 +43,19 @@ static CRITICAL_SECTION initLock;
 
 #if TCL_THREADS
 
-static struct Tcl_Mutex_ {
+/*
+ * Although CRITICAL_SECTIONs can be nested, we need to keep track
+ * of their lock counts for condition variables.
+ */
+
+typedef struct WMutex {
     CRITICAL_SECTION crit;
-} allocLock;
-static Tcl_Mutex allocLockPtr = &allocLock;
+    volatile LONG thread;
+    int counter;
+} WMutex;
+
+static struct WMutex allocLock;
+static WMutex *allocLockPtr = &allocLock;
 static int allocOnce = 0;
 
 #endif /* TCL_THREADS */
@@ -119,9 +128,12 @@ static DWORD tlsKey;
 
 typedef struct {
     Tcl_Mutex tlock;
-    CRITICAL_SECTION wlock;
+    WMutex wm;
 } allocMutex;
 #endif /* USE_THREAD_ALLOC */
+
+static void WMutexInit(WMutex *);
+static void WMutexDestroy(WMutex *);
 
 /*
  * The per thread data passed from TclpThreadCreate
@@ -216,8 +228,8 @@ TclpThreadCreate(
 
     EnterCriticalSection(&joinLock);
 
-    *idPtr = 0; /* must initialize as Tcl_Thread is a pointer and
-                 * on WIN64 sizeof void* != sizeof unsigned */
+    *idPtr = 0;		/* must initialize as Tcl_Thread is a pointer and
+			 * on WIN64 sizeof void* != sizeof unsigned */
 
 #if defined(_MSC_VER) || defined(__MSVCRT__)
     tHandle = (HANDLE) _beginthreadex(NULL, (unsigned)stackSize,
@@ -473,10 +485,10 @@ Tcl_GetAllocMutex(void)
 {
 #if TCL_THREADS
     if (!allocOnce) {
-	InitializeCriticalSection(&allocLock.crit);
+	WMutexInit(&allocLock);
 	allocOnce = 1;
     }
-    return &allocLockPtr;
+    return (Tcl_Mutex *) &allocLockPtr;
 #else
     return NULL;
 #endif
@@ -515,7 +527,7 @@ TclFinalizeLock(void)
 
 #if TCL_THREADS
     if (allocOnce) {
-	DeleteCriticalSection(&allocLock.crit);
+	WMutexDestroy(&allocLock);
 	allocOnce = 0;
     }
 #endif
@@ -531,6 +543,53 @@ TclFinalizeLock(void)
 
 #if TCL_THREADS
 
+static void
+WMutexInit(
+    WMutex *wmPtr)
+{
+    wmPtr->thread = 0;
+    wmPtr->counter = 0;
+    InitializeCriticalSection(&wmPtr->crit);
+}
+
+static void
+WMutexDestroy(
+    WMutex *wmPtr)
+{
+    DeleteCriticalSection(&wmPtr->crit);
+    assert(wmPtr->thread == 0 && wmPtr->counter == 0);
+}
+
+static void
+WMutexLock(
+    WMutex *wmPtr)
+{
+    LONG mythread = GetCurrentThreadId();
+
+    if (wmPtr->thread == mythread) {
+	// We owned the lock already, so it's recursive.
+	wmPtr->counter++;
+    } else {
+	// We don't own the lock, so we can safely lock it. Then we own it.
+	EnterCriticalSection(&wmPtr->crit);
+	wmPtr->thread = mythread;
+    }
+}
+
+static void
+WMutexUnlock(
+    WMutex *wmPtr)
+{
+    assert(wmPtr->thread == GetCurrentThreadId());
+    if (wmPtr->counter) {
+	// It's recursive
+	wmPtr->counter--;
+    } else {
+	wmPtr->thread = 0;
+	LeaveCriticalSection(&wmPtr->crit);
+    }
+}
+
 /* locally used prototype */
 static void		FinalizeConditionEvent(void *data);
 
@@ -553,9 +612,9 @@ static void		FinalizeConditionEvent(void *data);
 
 void
 Tcl_MutexLock(
-    Tcl_Mutex *mutexPtr)	/* The lock */
+    Tcl_Mutex *mutexPtr)	/* Really (WMutex **) */
 {
-    CRITICAL_SECTION *csPtr;
+    WMutex *wmPtr;
 
     if (*mutexPtr == NULL) {
 	TclpGlobalLock();
@@ -565,15 +624,15 @@ Tcl_MutexLock(
 	 */
 
 	if (*mutexPtr == NULL) {
-	    csPtr = (CRITICAL_SECTION *) Tcl_Alloc(sizeof(CRITICAL_SECTION));
-	    InitializeCriticalSection(csPtr);
-	    *mutexPtr = (Tcl_Mutex) csPtr;
+	    wmPtr = (WMutex *) Tcl_Alloc(sizeof(WMutex));
+	    WMutexInit(wmPtr);
+	    *mutexPtr = (Tcl_Mutex) wmPtr;
 	    TclRememberMutex(mutexPtr);
 	}
 	TclpGlobalUnlock();
     }
-    csPtr = *((CRITICAL_SECTION **)mutexPtr);
-    EnterCriticalSection(csPtr);
+    wmPtr = *((WMutex **)mutexPtr);
+    WMutexLock(wmPtr);
 }
 
 /*
@@ -594,11 +653,10 @@ Tcl_MutexLock(
 
 void
 Tcl_MutexUnlock(
-    Tcl_Mutex *mutexPtr)	/* The lock */
+    Tcl_Mutex *mutexPtr)	/* Really (WMutex **) */
 {
-    CRITICAL_SECTION *csPtr = *((CRITICAL_SECTION **)mutexPtr);
-
-    LeaveCriticalSection(csPtr);
+    WMutex *wmPtr = *((WMutex **)mutexPtr);
+    WMutexUnlock(wmPtr);
 }
 
 /*
@@ -620,13 +678,13 @@ Tcl_MutexUnlock(
 
 void
 TclpFinalizeMutex(
-    Tcl_Mutex *mutexPtr)
+    Tcl_Mutex *mutexPtr)	/* Really (WMutex **) */
 {
-    CRITICAL_SECTION *csPtr = *(CRITICAL_SECTION **)mutexPtr;
+    WMutex *wmPtr = *(WMutex **)mutexPtr;
 
-    if (csPtr != NULL) {
-	DeleteCriticalSection(csPtr);
-	Tcl_Free(csPtr);
+    if (wmPtr != NULL) {
+	WMutexDestroy(wmPtr);
+	Tcl_Free(wmPtr);
 	*mutexPtr = NULL;
     }
 }
@@ -660,9 +718,10 @@ Tcl_ConditionWait(
     const Tcl_Time *timePtr)	/* Timeout on waiting period */
 {
     WinCondition *winCondPtr;	/* Per-condition queue head */
-    CRITICAL_SECTION *csPtr;	/* Caller's Mutex, after casting */
+    WMutex *wmPtr;		/* Caller's Mutex, after casting */
     DWORD wtime;		/* Windows time value */
     int timeout;		/* True if we got a timeout */
+    int counter;		/* Caller's Mutex counter */
     int doExit = 0;		/* True if we need to do exit setup */
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
@@ -717,7 +776,7 @@ Tcl_ConditionWait(
 	}
 	TclpGlobalUnlock();
     }
-    csPtr = *((CRITICAL_SECTION **)mutexPtr);
+    wmPtr = *((WMutex **)mutexPtr);
     winCondPtr = *((WinCondition **)condPtr);
     if (timePtr == NULL) {
 	wtime = INFINITE;
@@ -752,7 +811,12 @@ Tcl_ConditionWait(
      * thread.
      */
 
-    LeaveCriticalSection(csPtr);
+    counter = wmPtr->counter;
+    wmPtr->counter = 0;
+    LONG mythread = GetCurrentThreadId();
+    assert(wmPtr->thread == mythread);
+    wmPtr->thread = 0;
+    LeaveCriticalSection(&wmPtr->crit);
     timeout = 0;
     while (!timeout && (tsdPtr->flags & WIN_THREAD_BLOCKED)) {
 	ResetEvent(tsdPtr->condEvent);
@@ -795,7 +859,9 @@ Tcl_ConditionWait(
     }
 
     LeaveCriticalSection(&winCondPtr->condLock);
-    EnterCriticalSection(csPtr);
+    EnterCriticalSection(&wmPtr->crit);
+    wmPtr->counter = counter;
+    wmPtr->thread = mythread;
 }
 
 /*
@@ -938,8 +1004,8 @@ TclpNewAllocMutex(void)
     if (lockPtr == NULL) {
 	Tcl_Panic("could not allocate lock");
     }
-    lockPtr->tlock = (Tcl_Mutex) &lockPtr->wlock;
-    InitializeCriticalSection(&lockPtr->wlock);
+    lockPtr->tlock = (Tcl_Mutex)&lockPtr->wm;
+    WMutexInit(&lockPtr->wm);
     return &lockPtr->tlock;
 }
 
@@ -949,10 +1015,11 @@ TclpFreeAllocMutex(
 {
     allocMutex *lockPtr = (allocMutex *) mutex;
 
-    if (!lockPtr) {
+    if (!lockPtr || !lockPtr->tlock) {
 	return;
     }
-    DeleteCriticalSection(&lockPtr->wlock);
+    lockPtr->tlock = NULL;
+    WMutexDestroy(&lockPtr->wm);
     free(lockPtr);
 }
 
